'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _request = require('request');

var _request2 = _interopRequireDefault(_request);

var _winston = require('winston');

var _winston2 = _interopRequireDefault(_winston);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Query interface for executing a request against the IBM Graph Service.
 *
 * Generic wrapper over request that handles for error checking nuances when
 * communicating with IBM Graph, also provides some context with `winston` and
 * provides meta information like the duration of the query to help out with
 * performance.
 *
 * @param {Object} params
 * @param {String} params.uri  - The uri to send the request to
 * @param {String} params.method - The HTTP method for sending the request
 * @param {Object} params.auth - An auth object containing user/pass creds.
 * @param {Object} params.json - Pass in an arbitrary payload in the request
 * @param {Object} params.headers  - Pass in headers for things like sessions
 * @returns {Object}
 */
var Query = function Query(_ref) {
  var uri = _ref.uri;
  var method = _ref.method;
  var auth = _ref.auth;
  var body = _ref.body;
  var headers = _ref.headers;
  var formData = _ref.formData;
  var _ref$debug = _ref.debug;
  var debug = _ref$debug === undefined ? false : _ref$debug;
  return new Promise(function (resolve, reject) {
    var start = Date.now();
    var options = {
      uri: uri,
      method: method,
      auth: auth,
      json: body,
      headers: _extends({
        'Content-Type': 'application/json',
        'Accept': 'application/json'
      }, headers)
    };

    if (formData) {
      options.formData = formData;
    }

    if (debug) {
      _winston2.default.info('' + Date.now(), 'request', JSON.stringify(options, null, 2));
    }

    (0, _request2.default)(options, function (error, res, body) {
      var end = Date.now();
      var duration = end - start;

      if (error) {
        reject(error);
        return;
      }

      /**
       * We have to wrap parsing our response in a try/catch because the service
       * may throw unexpected string errors like:
       *
       * 502 Bad Gateway: ...
       */
      var response = undefined;

      try {
        response = (typeof body === 'undefined' ? 'undefined' : _typeof(body)) === 'object' ? body : JSON.parse(body);
      } catch (e) {
        reject(body);
        return;
      }

      if (response.code) {
        reject(response);
        return;
      }

      resolve(_extends({}, response, {
        duration: duration
      }));

      if (debug) {
        _winston2.default.info('' + Date.now(), 'response', JSON.stringify(response, null, 2));
      }
    });
  });
};

exports.default = Query;